import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from tqdm import tqdm
from collections import defaultdict


# this script computes the distribution of smoothness/sharpness of the books;
# either considering all books, as well as considering different groups of books' reviews (by product)

# filein='/media/moreo/Volume/Datasets/Amazon/raw/Gift_Cards.json.gz'
# df = pd.read_json(filein, lines=True, compression='gzip')

read_meta = True


def prepare_vote_field(df):
    df['vote'] = df['vote'].fillna('0')
    df['vote'] = df['vote'].apply(lambda x: x.replace(',', ''))
    df['vote'] = pd.to_numeric(df['vote'])
    return df


def read_from_huge_json(filein):
    df = pd.read_json(filein, lines=True)
    df.drop(columns=[
        'verified', 'reviewTime', 'reviewerID', 'style', 'reviewerName', 'reviewText', 'summary', 'unixReviewTime',
        'image'
    ], inplace=True)

    df = prepare_vote_field(df)
    return df


def read_from_metadata(filein):
    df = pd.read_csv(filein)
    df['vote'] = pd.to_numeric(df['vote'])
    return df


def filter_by_vote(df, vote_threshold=1):
    df = df[df['vote'] >= vote_threshold]
    df.drop(columns=['vote'], inplace=True)
    return df


if read_meta:
    filein = '/media/moreo/Volume/Datasets/Amazon/meta/Books.csv'
    readfn = read_from_metadata
else:
    filein='/media/moreo/Volume/Datasets/Amazon/raw/Books.json'
    readfn = read_from_huge_json

votes_support=9

df = readfn(filein)

num_entries = len(df)
# df = prepare_vote_field(df)
df = filter_by_vote(df, vote_threshold=votes_support)
num_entries_with_vote = len(df)

unique_product_ids = df['asin'].unique()
num_products = len(unique_product_ids)

print(df.columns)
print(f'num rows {len(df)} (before vote-thresholding {num_entries}, after thresholding {num_entries_with_vote})')
print(f'num unique products {num_products}')


# df = df.groupby(df['asin'])

def not_smoothness(p):
    return 0.5 * sum((-p_prev + 2*p_i - p_next)**2 for p_prev, p_i, p_next in zip(p[:-2], p[1:-1], p[2:]))


# pass to dictionaries
df = df.reset_index()  # make sure indexes pair with number of rows

ids = df['asin'].values
overalls = df['overall'].values

allbooks_prev = np.histogram(overalls, bins=np.array([0, 1, 2, 3, 4, 5]) + 0.5, density=True)[0]
allbooks_sharpness = not_smoothness(allbooks_prev)
print(f'all books prev={allbooks_prev} has sharpness {allbooks_sharpness:.4f}')

import sys
sys.exit(0)

# Defining a dict
d = defaultdict(list)
for i, id in tqdm(enumerate(ids), total=len(ids), desc='passing to dictionary'):
    d[id].append(overalls[i])


by_review_support = []
by_review_support_label = []
for reviews_support in [50, 100, 1]:
    sharpness_all = []
    num_products_with_reviews = 0
    for product_id, ratings in tqdm(d.items(), total=len(d), desc='processing histograms'):
        # ratings = df[df["asin"] == product_id]["overall"].values
        n_ratings = len(ratings)
        if n_ratings >= reviews_support:
            # print(product_id, ratings)
            prev = np.histogram(ratings, bins=np.array([0, 1, 2, 3, 4, 5]) + 0.5, density=True)[0]
            sharpness = not_smoothness(prev)
            # print(prev, sharpness)
            sharpness_all.append(sharpness)
            num_products_with_reviews+=1
    by_review_support.append(sharpness_all)
    by_review_support_label.append(f'>{reviews_support}')

    print(f'#votes-support (min number of votes): {votes_support}')
    print(f'#reviews with >#votes-support: {num_entries_with_vote}/{num_entries}={100*num_entries_with_vote/num_entries:.2f}%')

    print(f'#reviews-support (min number of reviews): {reviews_support}')
    print(f'#products with >#reviews-support: {num_products_with_reviews}/{num_products}={100*num_products_with_reviews/num_products:.2f}%')

    q05 = np.percentile(sharpness_all, 5)
    q25 = np.percentile(sharpness_all, 25)
    q50 = np.percentile(sharpness_all, 50)
    q75 = np.percentile(sharpness_all, 75)
    q95 = np.percentile(sharpness_all, 95)
    print(f'{q05:.5f}\t{q25:.5f}\t{q50:.5f}\t{q75:.5f}\t{q95:.5f}')
    print(f'ave={np.mean(sharpness_all):.5f}')
    print(f'min={np.min(sharpness_all):.5f}')
    print(f'max={np.max(sharpness_all):.5f}')

#fig, ax = plt.subplots()
#ax.boxplot(by_review_support)
#ax.set_xticklabels(by_review_support_label)
#ax.set_ylabel("Sharpness")
#ax.set_xlabel("Distributions by number of reviews")
#plt.show()

